@@ -5,9 +5,10 @@ fn yields(x: jaq_interpret::Val, f: &str, ys: impl Iterator<Item = jaq_interpret
     ctx.insert_natives(jaq_core::core());
     ctx.insert_defs(jaq_std::std());
 
-    let (f, errs) = jaq_parse::parse(f, jaq_parse::main());
-    assert!(errs.is_empty());
-    ctx.yields(x, f.unwrap(), ys)
+    let f = jaq_syn::parse(f, |p| p.module(|p| p.term()))
+        .unwrap()
+        .conv(f);
+    ctx.yields(x, f, ys)
 }
 
 pub fn fail(x: Value, f: &str, err: jaq_interpret::Error) {
